@@ -1,3 +1,4 @@
+import logging
 import typing
 
 from corm.constants import CLUSTER_IPS, CLUSTER_PORT, PWN
@@ -12,6 +13,8 @@ TABLES = {}
 SESSIONS = {}
 CLUSTER = Cluster(CLUSTER_IPS, port=CLUSTER_PORT)
 
+logger = logging.getLogger(__name__)
+
 def obtain_session(keyspace: str):
     if keyspace in SESSIONS.keys():
         return SESSIONS[keyspace]
@@ -46,17 +49,67 @@ def register_table(table: typing.NamedTuple) -> None:
     table._corm_details = corm_details
 
 def sync_schema() -> None:
-    cmds_by_keyspace = {}
+    """
+    https://docs.datastax.com/en/dse/5.1/cql/cql/cql_using/useQuerySystemTable.html
+    """
+    keyspace_tables = {}
     for table_name, table in TABLES.items():
-        cmds = cmds_by_keyspace.get(table.keyspace, [])
-        cmds.append(table.as_create_keyspace_cql())
-        cmds.append(table.as_create_table_cql())
-        cmds_by_keyspace[table.keyspace] = cmds
+        tables = keyspace_tables.get(table.keyspace, [])
+        tables.append(table)
+        keyspace_tables[table.keyspace] = tables
 
-    for keyspace, cmds in cmds_by_keyspace.items():
-        session = obtain_session(keyspace)
-        for cmd in cmds:
-            session.execute(cmd)
+    for keyspace_name, tables in keyspace_tables.items():
+        obtain_session(keyspace_name).execute(table.as_create_keyspace_cql())
+        session = obtain_session(keyspace_name)
+        for table in tables:
+            COLUMN_CQL = f'''
+            SELECT
+                column_name, type
+            FROM
+                system_schema.columns
+            WHERE table_name = ?
+                AND keyspace_name = ?'''
+            stmt = obtain_session(keyspace_name).prepare(COLUMN_CQL)
+            existing_columns = {r.column_name: r.type for r in obtain_session(keyspace_name).execute(stmt, [table.table_name, keyspace_name])}
+            # Add Whole Table
+            if len(existing_columns.keys()) == 0:
+                logger.info(f'Creating Table[{table.table_name}]')
+                obtain_session(keyspace_name).execute(table.as_create_table_cql())
+
+            # Add Columns
+            elif len(table.field_names) > len(existing_columns.keys()) - 1:
+                column_updates = {}
+                for field_idx, field_name in enumerate(table.field_names):
+                    field_transliterator = table.field_transliterators[field_idx]
+                    if field_name in existing_columns.keys():
+                        if field_transliterator.cql_type.lower() != existing_columns[field_name].lower():
+                            raise NotImplementedError
+
+                    else:
+                        column_updates[field_name] = field_transliterator.cql_type
+
+                formatted_column_names = ', '.join(sorted(column_updates.keys()))
+                formatted_column_definitions = ',\n'.join([' '.join([c_name, c_type]) for c_name, c_type in column_updates.items()])
+                logger.info(f'Altering Table[{table.table_name}]. Adding Columns[{formatted_column_names}]')
+                ALTER_CQL = f'''
+ALTER TABLE
+    {keyspace_name}.{table.table_name}
+ADD ({formatted_column_definitions})
+'''
+                obtain_session(keyspace_name).execute(ALTER_CQL)
+
+            # Delete Columns
+            elif len(table.field_names) < len(existing_columns.keys()) - 1:
+                columns_to_be_removed = [key for key in existing_columns.keys() if not key in table.field_names and key != 'guid']
+                formatted_column_names = ', '.join(sorted(columns_to_be_removed))
+                ALTER_CQL = f'''
+ALTER TABLE
+    {keyspace_name}.{table.table_name}
+DROP ({formatted_column_names})
+'''
+                obtain_session(keyspace_name).execute(ALTER_CQL)
+                import pdb; pdb.set_trace()
+                pass
 
 def insert(corm_objects: typing.List[typing.Any]) -> None:
     keyspace = corm_objects[0]._corm_details.keyspace
